/*
	Setup:
		$ npm install -D typescript
		$ npm install @babylonjs/core @babylonjs/materials @babylonjs/loaders
*/

import { Vector3 } from "@babylonjs/core/Maths/math.vector.js";
import { FreeCamera } from "@babylonjs/core/Cameras/freeCamera.js";
import { HemisphericLight } from "@babylonjs/core/Lights/hemisphericLight.js";
import { Scene } from "@babylonjs/core/scene.js";
import { Engine } from "@babylonjs/core/Engines/engine.js";

import { AbstractMesh } from "@babylonjs/core";
import { SceneLoader } from "@babylonjs/core/Loading/sceneLoader.js";

// Neither of these seem to help.
//import "@babylonjs/loaders";
//import "@babylonjs/loaders/OBJ/objFileLoader";

function assert (condition: unknown, msg?: string): asserts condition {
    if (!condition) throw new Error(msg);
}

async function loadMeshes(scene: Scene, path: string): Promise<AbstractMesh[]|null> {
	console.log(`Loading ${path}`);
	let promise = SceneLoader.ImportMeshAsync(null, path, undefined, scene)
	.then( (result: any) => {
		console.log(`Finished loading ${path}`);
		return result.meshes;
	})
	.catch( () => {
		console.log(`Error loading ${path}`);
		return null;
	});
	return promise;
}

const canvas = document.getElementById("renderCanvas");
assert(canvas instanceof HTMLCanvasElement);

const engine = new Engine(canvas);
var scene = new Scene(engine);
var camera = new FreeCamera("camera1", new Vector3(0, 5, -10), scene);
camera.setTarget(Vector3.Zero());
camera.attachControl(canvas, true);
var light = new HemisphericLight("light1", new Vector3(0, 1, 0), scene);
light.intensity = 0.7;
		
engine.runRenderLoop(() => {
	scene.render();
});

// Attempt to load an .obj file ...
loadMeshes(scene, 'test.obj');
